@@ -81,37 +81,58 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
   }
   
   if (is.null(datasets)) datasets <- unique(object$source)
+
   
-  if (!is.null(unlist(object[['species']][['speciesIn']]))) {
+  if (!is.null(unlist(object[['species']][['speciesIn']]))) speciespreds <- TRUE
+  else speciespreds <- FALSE
     
-    speciespreds <- TRUE
+    if (predictor) {
+      
+      intercepts <- TRUE
+      covariates <- object$spatCovs$name
+      spatial <- TRUE
+      
+    }
+    
+    if (intercepts) {
+      
+      if (!is.null(marks)) {
+        
+        marks_intercepts <- paste0(marks,'_intercept')
+        
+        if (identical(rownames(object$summary.fixed)[rownames(object$summary.fixed) %in% mark_intercepts], character(0))) mark_intercepts <- NULL
+        
+        
+      }
+      
+      if (!is.null(datasets)) {
+        
+        intercept_terms <- paste0(datasets, '_intercept')
+        
+        if (identical(rownames(object$summary.fixed)[rownames(object$summary.fixed) %in% intercept_terms], character(0))) intercept_terms <- NULL
+        
+      }
+      
+      
+    } 
+    else {
+      
+      intercept_terms <- NULL
+      marks_intercepts <- NULL
+      
+    }
     
     if (is.null(species)) speciesin <- unique(unlist(object[['species']][['speciesIn']]))
     else speciesin <- species
     if (!all(species %in% unique(unlist(object[['species']][['speciesIn']])))) stop('Species provided not in model.')
     
-  }
-  else {
     
-    speciespreds <- FALSE
-    if (intercepts) {
-    
-      if (!is.null(marks)) marks_intercepts <- paste0(marks,'_intercept')
-      
-      if (!is.null(datasets)) intercept_terms <- paste0(datasets, '_intercept')
-      
-    }
-    
-  }
   
-  if (!intercepts) {
+  if (is.null(object$spatCovs$covariateFormula)) {
     
-    intercept_terms <- NULL
-    marks_intercepts <- NULL
-    
-  }
-  
   if (!all(covariates%in%row.names(object$summary.fixed)) && !all(as.vector(outer(paste0(unlist(object[['species']][['speciesIn']]),'_'), covariates, FUN = 'paste0'))%in%row.names(object$summary.fixed))) stop("Covariates provided not in model.")
+
+  }
   
   if (is.null(formula) && !intercepts && !spatial && is.null(covariates) && !temporal && !biasfield && !predictor) stop("Please provide either a formula or components of a formula to be predicted.")
   
@@ -127,6 +148,24 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
     else data <- inlabru::fm_int(mesh, format = format)
   }
   
+  if (speciespreds) {
+  
+  if (object[['species']][['speciesEffects']][['Intercepts']]) {
+    
+    data <- fm_cprod(data, data.frame(speciesIndexREMOVE = 1:length(unique(unlist(object$species$speciesIn)))))
+    names(data)[names(data) == 'speciesIndexREMOVE'] <- object[['species']][['speciesVar']]
+    
+  }
+    
+  if (object$spatial$species == 'replicate') {
+      
+  if (!object[['species']][['speciesVar']] %in% names(data)) data <- fm_cprod(data, data.frame(speciesSpatialGroup = 1:length(unique(unlist(object$species$speciesIn)))))
+  else data$speciesSpatialGroup <- data[[object[['species']][['speciesVar']]]]
+    
+  }
+    
+  }
+  
   if (!any(names(data) %in% object$spatCovs$name)) {
     
     for (spatCov in object$spatCovs$name) {
@@ -136,8 +175,22 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
       
       data[, covIndex] <- inlabru::eval_spatial(where =  data, 
                                                 data = get('spatialcovariates', 
-                                                envir = object$spatCov$env),
+                                                envir = object$spatCov$env)[spatCov],
                                                 layer = spatCov)
+      
+      if (any(is.na( data[, covIndex]))) {
+        
+        for (indFix in covIndex) {
+        
+        data[[indFix]] <- inlabru::bru_fill_missing(where =  data, 
+                                                      data = get('spatialcovariates', 
+                                                             envir = object$spatCov$env)[spatCov],
+                                                      layer = spatCov,
+                                                      values = data[[indFix]])
+        
+        }
+        
+      }
         
       
     }
@@ -189,7 +242,7 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
       if (!all(paste0(datasets,'_spatial') %in% names(object$summary.random))) stop('Spatial effects not provided in intModel.')
       else spatial_obj <- paste0(datasets, '_spatial')
       
-      time_formula <- paste0(fun,'(',paste0(c(covariates, biasnames, intercept_terms, spatial_obj), collapse = ' + '),')')
+      time_formula <- paste0(fun,'(',paste0(c(covariates, biasnames, intercept_terms, spatial_obj, intercepts_species), collapse = ' + '),')')
 
       formula <- formula(paste('~',paste0('data.frame(', time_variable,' = ', time_variable, ',formula =', time_formula,')')))
 
@@ -222,7 +275,7 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
       
     }
     
-    if (speciespreds && ! predictor) {
+    if (speciespreds) {
       
       int[['speciesPredictions']] <- vector(mode = 'list', length(speciesin))
       names(int[['speciesPredictions']]) <- speciesin
@@ -241,22 +294,19 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
           
           if (!object[['species']][['speciesEffects']][['Intercepts']]) {
             
-            species_int <- paste0(spec,'_intercept')
+            intercepts_species <- paste0(spec,'_intercept')
             
           } else if (object[['species']][['speciesEffects']][['Intercepts']]) {
               
-            species_int <- paste0(object[['species']][['speciesVar']], '_intercepts')
-            data <- fm_cprod(data, data.frame(speciesIndexREMOVE = 1:length(unique(unlist(object$species$speciesIn)))))
-            names(data)[names(data) == 'speciesIndexREMOVE'] <- object[['species']][['speciesVar']]
+            intercepts_species <- paste0(object[['species']][['speciesVar']], '_intercepts')
             
-            
-            } else species_int <- NULL
+            } else intercepts_species <- NULL
         
         }
-        else species_int <- NULL
+        else intercepts_species <- NULL
         
         if (spatial) {
-          
+          ##Fix this
           allSpat <- c(paste0(spec, '_spatial'),
                        paste0(spec, '_', names(object$dataType), '_spatial'),
                        'speciesShared')
@@ -267,9 +317,19 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
           }
         else species_spat <- NULL
         
-        species_formula <- formula(paste0('~', fun, '(', paste0(c(species_covs, species_int, species_spat), collapse = ' + '),')'))
+        species_formula <- formula(paste0('~', fun, '(', paste0(c(species_spat, species_covs, intercepts_species, intercept_terms), collapse = ' + '),')'))
         
-        int[[1]][[spec]] <- predict(object, data, formula = species_formula, ...)
+        if (any(c('speciesSpatialGroup', object[['species']][['speciesVar']]) %in% names(data))) {
+          
+          spInd <- object$species$speciesTable[object$species$speciesTable$species == spec,]$index
+          which <- names(data)[names(data) %in% c('speciesSpatialGroup', object[['species']][['speciesVar']]) ][1]
+          dataSP <- data[data[[which]] == spInd,]
+          
+          
+        }
+        else dataSP <- data
+        
+        int[[1]][[spec]] <- predict(object, dataSP, formula = species_formula, ...)
       
         }
       
@@ -299,11 +359,18 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
       
       } 
       }
-    else spatial_obj <- NULL
+    else {
+      
+      spatial_obj <- NULL
+      marks_spatial <- NULL
+      
+    }
     
     if (predictor) formula_components <- c(row.names(object$summary.fixed), names(object$summary.random)[!names(object$summary.random) %in% paste0(object[['source']], '_biasField')])
     else formula_components <- c(covariates, intercept_terms, spatial_obj, marks_spatial, marks_intercepts)
     
+    if (!is.null(object$spatCovs$biasFormula)) formula_components <- formula_components[!formula_components %in% c('Bias__Effects__Comps', paste0(unique(object$species$speciesIn),'_Bias__Effects__Comp'))]
+    
     if (all(is.null(formula_components))) stop('Please specify at least one of: covariates, spatial, intercepts or biasfield.')
     
     formula <- as.formula(paste0('~ ',as.character(fun),'(',paste(formula_components, collapse = ' + '),')'))
@@ -321,6 +388,19 @@ predict.bruSDM <- function(object, data = NULL, formula = NULL, mesh = NULL,
     
     class(object) <- c('bru','inla','iinla')
     int <- predict(object, newdata = data, formula = formula, ...)
+    if (any(c('speciesSpatialGroup', object[['species']][['speciesVar']]) %in% names(int))) {
+      int[['..speciesPlotVar..']] <- NA
+      which <- names(int)[names(int) %in% c('speciesSpatialGroup', object[['species']][['speciesVar']])][1]
+      
+      for (spec in speciesin) {
+      
+      spInd <- object$species$speciesTable[object$species$speciesTable$species == spec,]$index
+      try(int[int[[which]] == spInd,][[which]] <- spec, silent = TRUE)
+      try(int[int[[which]] == spec,][['..speciesPlotVar..']] <- spec, silent = TRUE)
+      
+      }
+      
+    }
     int <- list(int)
     names(int) <- 'predictions'
     class(int) <- c('bruSDM_predict', class(int))
@@ -460,8 +540,8 @@ plot.bruSDM_predict <- function(x,
     #names(x[[1]]@data)[names(x[[1]]@data) == temporalName] <- '..temporal_variable_index..'
     x[[1]]$..temporal_variable_index.. <- as.character(data.frame(x[[1]])[, temporalName])
 
-    if (inherits(x[[1]], 'sf')) plot_obj <- geom_sf(data = x[[1]], aes_string(col = whattoplot))
-    else plot_obj <- inlabru::gg(x[[1]], aes_string(col = whattoplot))
+    if (inherits(x[[1]], 'sf')) plot_obj <- geom_sf(data = x[[1]], aes(col = .data[[whattoplot]]))
+    else plot_obj <- inlabru::gg(x[[1]], aes(col = .data[[whattoplot]]))
     
     ##Would be nice to get full temporal variable names in here ...
     plot_grid <- ggplot() + plot_obj + facet_wrap(~ ..temporal_variable_index..) + ggtitle('Plot of the temporal predictions')
@@ -482,8 +562,8 @@ plot.bruSDM_predict <- function(x,
       if (nameObj ==  'speciesPredictions') title <- ggtitle(paste('Plot of predictions for', object))
       else title <- ggtitle(paste('Plot of bias field for', object))
       
-      if (inherits(x[[nameObj]][[object]], 'sf')) plot_obj <- geom_sf(data = x[[nameObj]][[object]], aes_string(col = whattoplot))
-      else plot_obj <- inlabru::gg(x[[nameObj]][[object]], aes_string(col = whattoplot))
+      if (inherits(x[[nameObj]][[object]], 'sf')) plot_obj <- geom_sf(data = x[[nameObj]][[object]], aes(col = .data[[whattoplot]]))
+      else plot_obj <- inlabru::gg(x[[nameObj]][[object]], aes(col = .data[[whattoplot]]))
 
       all_plots[[object]] <- ggplot() + plot_obj + title + colours
       
@@ -518,13 +598,16 @@ plot.bruSDM_predict <- function(x,
       #title <- ggtitle(paste('Plot of',stat,'for',plotname))
       title <- ggtitle('Plot of predictions')
       
-      if (inherits(x[[plotname]], 'sf')) prediction <- geom_sf(data = x[[plotname]], aes_string(col = stat))
-      else prediction <- inlabru::gg(x[[plotname]], aes_string(col = stat))
+      if ('..speciesPlotVar..' %in% names(x[[plotname]])) speciesTerm <- facet_wrap(~..speciesPlotVar..)
+      else speciesTerm <- NULL
       
-      if (!plot) prediction_list[[stat]] <- ggplot() + prediction
+      if (inherits(x[[plotname]], 'sf')) prediction <- geom_sf(data = x[[plotname]], aes(col = .data[[stat]]))
+      else prediction <- inlabru::gg(x[[plotname]], aes(col = .data[[stat]]))
+      
+      if (!plot) prediction_list[[stat]] <- ggplot() + prediction + speciesTerm
 
       
-      plot_list[[stat]] <- ggplot() + prediction + title + colours
+      plot_list[[stat]] <- ggplot() + prediction + title + colours + speciesTerm
       
     }
     
