@@ -19,7 +19,7 @@ setClass('modSpecies_predict')
 #' @param datasets Names of the datasets to include intercept and spatial term.
 #' @param species Names of the species to predict. Default of \code{NULL} results in all species being predicted.
 #' @param bias Logical include bias field in prediction. Defaults to \code{FALSE}.
-#' @param biasnames Names of the datasets to include bias term. Defaults to \code{NULL}. Note: the chosen dataset needs to be run with a bias field first; this can be done using \code{.$addBias} with the object produced by \code{\link{intModel}}.
+#' @param biasnames Names of the datasets to include bias term. Defaults to \code{NULL}. Note: the chosen dataset needs to be run with a bias field first; this can be done using \code{.$addBias} with the object produced by \code{\link{startSpecies}}.
 #' @param predictor Should all terms (except the bias terms) included in the linear predictor be used in the predictions. Defaults to \code{FALSE}.
 #' @param fun Function used to predict. Set to \code{'linear'} if effects on the linear scale are desired.
 #' @param ... Additional arguments used by the inlabru \code{predict} function.
@@ -93,7 +93,13 @@ predict.modSpecies <- function(object, data = NULL, formula = NULL, mesh = NULL,
     
     if (!is.null(object$spatCovs$biasFormula)) covariates <- covariates[!covariates %in% labels(terms(object$spatCovs$biasFormula))]
     
-    spatial <- TRUE
+    if(is.character(object$spatial$points) | is.character(object$spatial$species)) spatial <- TRUE
+    else {
+      
+      if (!object$spatial$points & !object$spatial$species) spatial <- FALSE
+      else spatial <- TRUE
+      
+    }
     
   }
   
@@ -159,7 +165,7 @@ predict.modSpecies <- function(object, data = NULL, formula = NULL, mesh = NULL,
     
   }
   
-  if (!any(names(data) %in% object$spatCovs$name)) {
+  if (!any(names(data) %in% c(object$spatCovs$name,  paste0(unique(unlist(object$species$speciesIn)), '_', object$spatCovs$name)))) {
     
     for (spatCov in object$spatCovs$name) {
       
@@ -325,6 +331,7 @@ predict.modSpecies <- function(object, data = NULL, formula = NULL, mesh = NULL,
         int <- list(mapply(function(x, seq) {
           
           pred <- x[x[[object$species$speciesVar]] == seq,]
+          if ('speciesSpatialGroup' %in% names(pred)) pred <- pred[pred[['speciesSpatialGroup']] == seq,]
           list(pred)
           
           
@@ -340,15 +347,23 @@ predict.modSpecies <- function(object, data = NULL, formula = NULL, mesh = NULL,
     
     if (spatial) {
       
+      if (object$spatial$points == 'correlate') {
+        
+        if (any(object$dataType == "Present absence")) data$._dataset_index_var_. <- which(object$dataType == "Present absence")[1]
+        else data$._dataset_index_var_. <- 1
+      
+        }
         
         if ('shared_spatial' %in% names(object$summary.random))  spatial_obj <- 'shared_spatial'
         else
           if (object$spatial$points == 'copy') spatial_obj <- paste0(object$source[1], '_spatial')
           else
-            if (!all(paste0(datasets,'_spatial') %in% names(object$summary.random))) stop('Spatial effects not provided in intModel.')
+            if (!all(paste0(datasets,'_spatial') %in% names(object$summary.random))) stop('Spatial effects not provided in startSpecies.')
           else spatial_obj <- paste0(datasets, '_spatial')
           }
          else spatial_obj <- NULL
+    
+    
 
     
     if (predictor) formula_components <- c(row.names(object$summary.fixed), names(object$summary.random)[!names(object$summary.random) %in% paste0(object[['source']], '_biasField')])
@@ -425,7 +440,7 @@ predict.modSpecies <- function(object, data = NULL, formula = NULL, mesh = NULL,
 #'  mesh$crs <- proj
 #'  
 #'  #Set model up
-#'  organizedData <- intModel(data, Mesh = mesh, Coordinates = c('X', 'Y'),
+#'  organizedData <- startSpecies(data, Mesh = mesh, Coordinates = c('X', 'Y'),
 #'                              Projection = proj, responsePA = 'Present')
 #'  
 #'   ##Run the model
