@@ -27,22 +27,18 @@ PA$PAresp <- sample(x = c(0,1), size = nrow(PA), replace = TRUE)
 PA$trial <- sample(x = c(1,2,3), size = nrow(PA), replace = TRUE)
 PA$pointcov <- runif(n = nrow(PA))
 PA$binommark <- sample(x = 0:1, size = nrow(PA), replace = TRUE)
-PA$marktrial <- sample(x = 2:5, size = nrow(PA), replace = TRUE)
+PA$marktrial <- sample(x = 1:3, size = nrow(PA), replace = TRUE)
 PA$species <- sample(x = c('bird1', 'bird2'), nrow(PA), replace = TRUE)
 PA$temp <- sample(x = c(1,2), nrow(PA), replace = TRUE)
 mesh <- INLA::inla.mesh.2d(boundary = INLA::inla.sp2segment(SpatialPoly), 
                            max.edge = 2, crs = inlabru::fm_crs(projection))
 #iPoints <- inlabru::ipoints(samplers = SpatialPoly, domain = mesh)
 iPoints <- inlabru::fm_int(samplers = SpatialPoly, domain = mesh)
-##Make PA a data.frame object
-PA$long <- st_coordinates(PA)[,1]
-PA$lat <- st_coordinates(PA)[,2]
-st_geometry(PA) <- NULL
-PA <- data.frame(PA)
+
 
 spData <- list(PO, PA)
 
-test_that('The internal function makeData returns a list of SpatialPointDataFrame objects as
+test_that('The internal function makeData returns a list of sf objects as
           well as the relevant metadata to be used in the integrated model.', {
     
     Check$makeData(datapoints = spData, datanames = c('PO', 'PA'),
@@ -58,13 +54,13 @@ test_that('The internal function makeData returns a list of SpatialPointDataFram
     ##Should create a placeholder variable for the poresp + 
     #should keep marks +
     #should create new variables for the multinomial marks.
-    expect_setequal(names(Check$Data$PO[[1]]), c("poresp", "numvar", "factvar", 'temp', 'geometry',
+    expect_setequal(names(Check$Data$PO[[1]]), c("poresp", "numvar", "factvar", 'temp', 'geometry', '._dataset_index_var_.',
                                                     "species", "factvar_phi", "factvar_response", 'speciesINDEX_VAR'))
     expect_true((all(Check$Data$PO[[1]]$factvar_phi == 1)))
     expect_true((all(Check$Data$PO[[1]]$factvar_response == 1)))
     expect_true(class(Check$Data$PO[[1]]$factvar) == 'character')
     
-    expect_setequal(names(Check$Data$PA[[1]]), c("PAresp", "trial", "binommark", 'temp', 'geometry',
+    expect_setequal(names(Check$Data$PA[[1]]), c("PAresp", "trial", "binommark", 'temp', 'geometry', '._dataset_index_var_.',
                                                     "marktrial", "species", "pointcov", 'speciesINDEX_VAR'))
     #Family for PO should be:
     # cp for the points;
@@ -167,7 +163,7 @@ test_that('makeFormulas is able to make the correct formulas for the different p
             Check$makeFormulas(spatcovs = 'spatcovs', speciesname = 'species', markintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
                                paresp = 'PAresp', countresp = 'counts', marksspatial = TRUE, speciesspatial = 'individual',
                                marks = c('numvar', 'factvar', 'binommark'), temporalname = 'temp', speciesindependent = FALSE,
-                               spatial = 'shared', intercept = TRUE, pointcovs = 'pointcov')
+                               spatial = 'shared', intercept = FALSE, pointcovs = 'pointcov', biasformula = NULL, covariateformula = NULL)
             
             expect_setequal(names(Check$Formulas), c('PO', 'PA'))
             expect_setequal(names(Check$Formulas$PO), c('fish1','fish2'))
@@ -194,60 +190,60 @@ test_that('makeFormulas is able to make the correct formulas for the different p
             expect_setequal(Check$Formulas$PO$fish1$geometry$RHS,
                           c("fish1_spatcovs", "fish1_PO_spatial", "shared_spatial", "fish1_intercept"))
             expect_setequal(Check$Formulas$PO$fish1$numvar$RHS,
-                            c("fish1_spatcovs", "numvar_intercept", "numvar_spatial"))
+                            c("fish1_spatcovs", "numvar_intercept", "numvar_spatial", 'PO_numvar_spatial'))
             expect_setequal(Check$Formulas$PO$fish1$factvar_response$RHS,
                             c("fish1_spatcovs",
-                              "factvar_spatial", "factvar", "factvar_phi"))
+                              "factvar_spatial", "factvar", "factvar_phi", 'PO_factvar_response_spatial'))
             
             expect_setequal(Check$Formulas$PA$bird1$PAresp$RHS,
                            c("bird1_spatcovs", "bird1_PA_spatial", "shared_spatial", "bird1_intercept", "pointcov"))
             expect_setequal(Check$Formulas$PA$bird2$binommark$RHS,
-                            c("bird2_spatcovs", "binommark_spatial", "binommark_intercept"))
+                            c("bird2_spatcovs", "binommark_spatial", "binommark_intercept", 'PA_binommark_spatial'))
             
             ##Change terms
              #Set spatial and intercept to FALSE
             Check$makeFormulas(spatcovs = 'spatcovs', speciesname = 'species', temporalname = 'temp', speciesintercept = NULL, speciesenvironment = TRUE,
                                paresp = 'PAresp', countresp = 'counts', marksspatial = FALSE, speciesspatial = NULL,
                                marks = c('numvar', 'factvar', 'binommark'), markintercept = FALSE, speciesindependent = FALSE,
-                               spatial = NULL, intercept = FALSE, pointcovs = 'pointcov')
+                               spatial = NULL, intercept = FALSE, pointcovs = 'pointcov', biasformula = NULL, covariateformula = NULL)
           
             
             expect_setequal(Check$Formulas$PO$fish1$geometry$RHS,
                             c("fish1_spatcovs"))
             expect_setequal(Check$Formulas$PO$fish1$numvar$RHS,
-                            c("fish1_spatcovs"))
+                            c("fish1_spatcovs", 'PO_numvar_spatial'))
             expect_setequal(Check$Formulas$PO$fish1$factvar_response$RHS,
-                            c("fish1_spatcovs", "factvar", "factvar_phi"))
+                            c("fish1_spatcovs", "factvar", "factvar_phi", 'PO_factvar_response_spatial'))
             
             expect_setequal(Check$Formulas$PA$bird1$PAresp$RHS,
                             c("bird1_spatcovs", "pointcov"))
             expect_setequal(Check$Formulas$PA$bird2$binommark$RHS,
-                            c("bird2_spatcovs"))
+                            c("bird2_spatcovs", 'PA_binommark_spatial'))
             
             ##Change terms
             #Set spatcovs to NULL
             Check$makeFormulas(spatcovs = NULL, speciesname = 'species', marksspatial = TRUE, speciesspatial = 'individual',
                                paresp = 'PAresp', countresp = 'counts', markintercept = FALSE, speciesindependent = FALSE,
                                marks = c('numvar', 'factvar', 'binommark'), temporalname = 'temp', speciesintercept = FALSE, speciesenvironment = FALSE,
-                               spatial = 'shared', intercept = TRUE, pointcovs = 'pointcov')
+                               spatial = 'shared', intercept = FALSE, pointcovs = 'pointcov', biasformula = NULL, covariateformula = NULL)
             
             expect_setequal(Check$Formulas$PO$fish1$geometry$RHS,
                             c("fish1_PO_spatial", "shared_spatial", "fish1_intercept"))
             expect_setequal(Check$Formulas$PO$fish1$numvar$RHS,
-                            c("numvar_spatial"))
+                            c("numvar_spatial", 'PO_numvar_spatial'))
             expect_setequal(Check$Formulas$PO$fish1$factvar_response$RHS,
-                            c("factvar_spatial", "factvar", "factvar_phi"))
+                            c("factvar_spatial", "factvar", "factvar_phi", 'PO_factvar_response_spatial'))
             
             expect_setequal(Check$Formulas$PA$bird1$PAresp$RHS,
                             c("bird1_PA_spatial", "shared_spatial", "bird1_intercept", "pointcov"))
             expect_setequal(Check$Formulas$PA$bird2$binommark$RHS,
-                            c("binommark_spatial"))
+                            c("binommark_spatial", 'PA_binommark_spatial'))
             
             ##Try copy model
             Check$makeFormulas(spatcovs = NULL, speciesname = 'species', marksspatial = TRUE, speciesspatial = 'individual',
                                      paresp = 'PAresp', countresp = 'counts', markintercept = FALSE, speciesindependent = FALSE,
                                      marks = c('numvar', 'factvar', 'binommark'), temporalname = 'temp', speciesintercept = FALSE, speciesenvironment = TRUE,
-                                     spatial = 'copy', intercept = TRUE, pointcovs = 'pointcov')
+                                     spatial = 'copy', intercept = FALSE, pointcovs = 'pointcov', biasformula = NULL, covariateformula = NULL)
             
             expect_setequal(Check$Formulas$PO$fish2$geometry$RHS,
                             c('PO_spatial',"fish2_PO_spatial", "fish2_intercept"))
@@ -262,7 +258,7 @@ test_that('makeFormulas is able to make the correct formulas for the different p
             Check$makeFormulas(spatcovs = 'spatcovs', speciesname = 'species', markintercept = TRUE, speciesintercept = TRUE, speciesenvironment = TRUE,
                                paresp = 'PAresp', countresp = 'counts', marksspatial = TRUE, speciesspatial = 'individual',
                                marks = c('numvar', 'factvar', 'binommark'), temporalname = 'temp', speciesindependent = FALSE,
-                               spatial = 'shared', intercept = TRUE, pointcovs = 'pointcov')
+                               spatial = 'shared', intercept = FALSE, pointcovs = 'pointcov', biasformula = NULL, covariateformula = NULL)
             
             expect_setequal(Check$Formulas$PO$fish2$geometry$RHS,
                             c('shared_spatial',"fish2_PO_spatial", "species_intercepts", 'fish2_spatcovs'))
@@ -273,22 +269,65 @@ test_that('makeFormulas is able to make the correct formulas for the different p
             expect_setequal(Check$Formulas$PA$bird1$PAresp$RHS,
                             c("shared_spatial", "species_intercepts", "bird1_PA_spatial", 'bird1_spatcovs', 'pointcov'))
             
+            #Check biasFormula
+            Check$makeFormulas(spatcovs = 'spatcovs', speciesname = 'species', markintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
+                               paresp = 'PAresp', countresp = 'counts', marksspatial = TRUE, speciesspatial = 'individual',
+                               marks = c('numvar', 'factvar', 'binommark'), temporalname = 'temp', speciesindependent = FALSE,
+                               spatial = 'shared', intercept = FALSE, pointcovs = 'pointcov', biasformula = ~ biasVar, covariateformula = NULL)
+            
+            #Bias only in PO
+            expect_setequal(Check$Formulas$PO$fish2$geometry$RHS,
+                            c("fish2_spatcovs", "shared_spatial", "fish2_intercept", "fish2_PO_spatial", "Bias__Effects__Comps"))
+            expect_setequal(Check$Formulas$PO$fish1$geometry$RHS,
+                            c("fish1_spatcovs", "shared_spatial", "fish1_intercept", "fish1_PO_spatial", "Bias__Effects__Comps"))
+            
+            expect_setequal(Check$Formulas$PA$bird1$PAresp$RHS,
+                            c("bird1_spatcovs", "shared_spatial", "bird1_intercept", "pointcov","bird1_PA_spatial"))
+            expect_setequal(Check$Formulas$PA$bird2$PAresp$RHS,
+                            c("bird2_spatcovs", "shared_spatial", "bird2_intercept", "pointcov","bird2_PA_spatial"))
+            
+            #Check covariateFormula
+            Check$makeFormulas(spatcovs = 'spatcovs', speciesname = 'species', markintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
+                               paresp = 'PAresp', countresp = 'counts', marksspatial = TRUE, speciesspatial = 'individual',
+                               marks = c('numvar', 'factvar', 'binommark'), temporalname = 'temp', speciesindependent = FALSE,
+                               spatial = 'shared', intercept = FALSE, pointcovs = 'pointcov', biasformula = NULL, covariateformula = ~ cov + I(cov^2))
+            
+            expect_setequal(Check$Formulas$PO$fish2$geometry$RHS,
+                            c("fish2_Fixed__Effects__Comps", "shared_spatial", "fish2_intercept", "fish2_PO_spatial"))
+            expect_setequal(Check$Formulas$PO$fish1$geometry$RHS,
+                            c("fish1_Fixed__Effects__Comps", "shared_spatial", "fish1_intercept", "fish1_PO_spatial"))
+            
+            expect_setequal(Check$Formulas$PA$bird1$PAresp$RHS,
+                            c("bird1_Fixed__Effects__Comps", "shared_spatial", "bird1_intercept", "pointcov","bird1_PA_spatial"))
+            expect_setequal(Check$Formulas$PA$bird2$PAresp$RHS,
+                            c("bird2_Fixed__Effects__Comps", "shared_spatial", "bird2_intercept", "pointcov","bird2_PA_spatial"))
+            
+            #Check replicate
+            Check$makeFormulas(spatcovs = 'spatcovs', speciesname = 'species', markintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
+                               paresp = 'PAresp', countresp = 'counts', marksspatial = TRUE, speciesspatial = 'replicate',
+                               marks = NULL, temporalname = 'temp', speciesindependent = FALSE,
+                               spatial = 'correlate', intercept = FALSE, pointcovs = 'pointcov', biasformula = NULL, covariateformula = NULL)
+            
+            expect_true(all(unlist(lapply(unlist(unlist(Check$Formulas, recursive = FALSE), recursive = FALSE), function(x) 'shared_spatial' %in% x$RHS))))
+            
+            
             })
 
 #Change back to original
 Check$makeFormulas(spatcovs = 'spatcovs', speciesname = 'species', marksspatial = TRUE, speciesintercept = TRUE, speciesenvironment = TRUE,
                    paresp = 'PAresp', countresp = 'counts', markintercept = TRUE, speciesspatial = 'individual',
-                   marks = c('numvar', 'factvar', 'binommark'), temporalname = 'temp', speciesindependent = FALSE,
-                   spatial = 'shared', intercept = TRUE, pointcovs = 'pointcov')
+                   marks = NULL, temporalname = 'temp', speciesindependent = FALSE,
+                   spatial = 'shared', intercept = FALSE, pointcovs = 'pointcov', biasformula = NULL, covariateformula = NULL)
 
 test_that('makeComponents is able to make the correct components for all the processes
           based on the predictors and spatial effects available.', {
             
-    comps <- Check$makeComponents(spatial = 'shared', intercepts = TRUE, datanames = c('PO','PA'), marksintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
+    comps <- Check$makeComponents(spatial = 'shared', intercepts = FALSE, datanames = c('PO','PA'), marksintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
                          marks = c('numvar', 'factvar', 'binommark'), temporalmodel = deparse(list(model = "ar1")), speciesindependent = FALSE,
                          multinomnames = 'factvar', pointcovariates = 'pointcov', marksspatial = TRUE, offsetname = NULL,
                          speciesname = 'species', covariatenames = 'spatcovs', temporalname = 'temp', speciesspatial = 'individual',
-                         covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel)
+                         covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel,
+                         biasformula = NULL, covariateformula = NULL, marksCopy = list(PO = c('numvar', 'factvar'), PA = 'binommark'))
     
     expect_setequal(comps,c("shared_spatial(main = geometry, model = shared_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",
                             "fish2_PO_spatial(main = geometry, model = fish2_PO_field)",                                                                   
@@ -297,7 +336,10 @@ test_that('makeComponents is able to make the correct components for all the pro
                             "bird1_PA_spatial(main = geometry, model = bird1_PA_field)",                                                                   
                             "numvar_spatial(main = geometry, model = numvar_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))"  ,                                                                 
                             "factvar_spatial(main = geometry, model = factvar_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",                                                               
-                            "binommark_spatial(main = geometry, model = binommark_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",                                                           
+                            "binommark_spatial(main = geometry, model = binommark_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",
+                            "PO_numvar_spatial(main = geometry, copy = \"shared_spatial\")", 
+                            "PO_factvar_spatial(main = geometry, copy = \"shared_spatial\")",
+                            "PA_binommark_spatial(main = geometry, copy = \"shared_spatial\")",
                             "fish2_spatcovs(main = fish2_spatcovs, model = \"numeric\")",                                                               
                             "fish1_spatcovs(main = fish1_spatcovs, model = \"numeric\")",                                                               
                             "bird2_spatcovs(main = bird2_spatcovs, model = \"numeric\")",                                                               
@@ -314,11 +356,12 @@ test_that('makeComponents is able to make the correct components for all the pro
     
     ## Change arguments
     #spatial and intercepts == FALSE
-    comps2 <- Check$makeComponents(spatial = NULL, intercepts = FALSE, datanames = c('PO','PA'), speciesintercept = FALSE, speciesenvironment = TRUE,
+    comps2 <- Check$makeComponents(spatial = NULL, intercepts = FALSE, datanames = c('PO','PA'), speciesintercept = NULL, speciesenvironment = TRUE,
                                   marks = c('numvar', 'factvar', 'binommark'), marksspatial = FALSE, offsetname = NULL,
                                   multinomnames = 'factvar', pointcovariates = 'pointcov', marksintercept = FALSE, speciesindependent = FALSE,
                                   speciesname = 'species', covariatenames = 'spatcovs', speciesspatial = 'individual',
-                                  covariateclass = 'numeric', numtime =  2,  copymodel = Check$.__enclos_env__$private$copyModel)
+                                  covariateclass = 'numeric', numtime =  2,  copymodel = Check$.__enclos_env__$private$copyModel,
+                                  biasformula = NULL, covariateformula = NULL, marksCopy = list(PO = c('numvar', 'factvar'), PA = 'binommark'))
     
     expect_setequal(comps2,c("fish2_PO_spatial(main = geometry, model = fish2_PO_field)",                       
                              "fish1_PO_spatial(main = geometry, model = fish1_PO_field)",                       
@@ -331,13 +374,30 @@ test_that('makeComponents is able to make the correct components for all the pro
                              "pointcov",                                                                     
                              "factvar(main = factvar, model = \"iid\",constr = FALSE, fixed=TRUE)",          
                              "factvar_phi(main = factvar_phi, model = \"iid\", initial = -10, fixed = TRUE)"))
+    #Check replicate
+    compsrep <- Check$makeComponents(spatial = NULL, intercepts = FALSE, datanames = c('PO','PA'), speciesintercept = NULL, speciesenvironment = TRUE,
+                                   marks = c('numvar', 'factvar', 'binommark'), marksspatial = FALSE, offsetname = NULL,
+                                   multinomnames = 'factvar', pointcovariates = 'pointcov', marksintercept = FALSE, speciesindependent = FALSE,
+                                   speciesname = 'species', covariatenames = 'spatcovs', speciesspatial = 'replicate',
+                                   covariateclass = 'numeric', numtime =  2,  copymodel = Check$.__enclos_env__$private$copyModel,
+                                   biasformula = NULL, covariateformula = NULL, marksCopy = list(PO = c('numvar', 'factvar'), PA = 'binommark'))
+    
+    expect_setequal(compsrep,c("speciesShared(main = geometry, model = speciesField, group = speciesSpatialGroup, control.group = list(model = \"iid\"))",                       
+                             "fish2_spatcovs(main = fish2_spatcovs, model = \"numeric\")",                   
+                             "fish1_spatcovs(main = fish1_spatcovs, model = \"numeric\")",                   
+                             "bird2_spatcovs(main = bird2_spatcovs, model = \"numeric\")",                   
+                             "bird1_spatcovs(main = bird1_spatcovs, model = \"numeric\")",                   
+                             "pointcov",                                                                     
+                             "factvar(main = factvar, model = \"iid\",constr = FALSE, fixed=TRUE)",          
+                             "factvar_phi(main = factvar_phi, model = \"iid\", initial = -10, fixed = TRUE)"))
     
     #checkComponents with a copy model
-    compsCopy <- Check$makeComponents(spatial = 'copy', intercepts = FALSE, datanames = c('PO','PA'), speciesintercept = FALSE, speciesenvironment = TRUE,
+    compsCopy <- Check$makeComponents(spatial = 'copy', intercepts = FALSE, datanames = c('PO','PA'), speciesintercept = NULL, speciesenvironment = TRUE,
                          marks = c('numvar', 'factvar', 'binommark'), marksspatial = FALSE, offsetname = NULL,
                          multinomnames = 'factvar', pointcovariates = 'pointcov', marksintercept = FALSE, speciesindependent = FALSE,
                          speciesname = 'species', covariatenames = 'spatcovs', speciesspatial = 'individual', temporalname = NULL,
-                         covariateclass = 'numeric', numtime =  NULL,  copymodel = "list(beta = list(fixed = FALSE))")
+                         covariateclass = 'numeric', numtime =  NULL,  copymodel = "list(beta = list(fixed = FALSE))",
+                         biasformula = NULL, covariateformula = NULL, marksCopy = list(PO = c('numvar', 'factvar'), PA = 'binommark'))
     
     expect_setequal(compsCopy,c("PO_spatial(main = geometry, model = PO_field)",                                          
                              "PA_spatial(main = geometry, copy = \"PO_spatial\", hyper = list(beta = list(fixed = FALSE)))",
@@ -358,7 +418,8 @@ test_that('makeComponents is able to make the correct components for all the pro
                                   marks = c('numvar', 'factvar', 'binommark'), temporalmodel = deparse(list(model = "ar1")), speciesindependent = FALSE,
                                   multinomnames = 'factvar', pointcovariates = 'pointcov', marksspatial = TRUE, offsetname = NULL,
                                   speciesname = 'species', covariatenames = 'spatcovs', temporalname = 'temp', speciesspatial = 'individual',
-                                  covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel)
+                                  covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel,
+                                  biasformula = NULL, covariateformula = NULL, marksCopy = list(PO = c('numvar', 'factvar'), PA = 'binommark'))
     
     expect_setequal(compsRandom,
                     c("shared_spatial(main = geometry, model = shared_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",
@@ -373,8 +434,13 @@ test_that('makeComponents is able to make the correct components for all the pro
                             "fish1_spatcovs(main = fish1_spatcovs, model = \"numeric\")",                                                               
                             "bird2_spatcovs(main = bird2_spatcovs, model = \"numeric\")",                                                               
                             "bird1_spatcovs(main = bird1_spatcovs, model = \"numeric\")",                                                               
-                            "pointcov",                                                                                                                 
-                            'species_intercepts(main = species, model = "iid")',
+                            "pointcov",        
+                            "PO_intercept(1)",
+                            "PA_intercept(1)",
+                            "PO_factvar_spatial(main = geometry, copy = \"shared_spatial\")" ,
+                            "PO_numvar_spatial(main = geometry, copy = \"shared_spatial\")",
+                            "PA_binommark_spatial(main = geometry, copy = \"shared_spatial\")",
+                            "species_intercepts(main = species, model = \"iid\", constr = TRUE, hyper = list(prec = list(prior = \"loggamma\", param = c(1, 5e-05))))",
                             "factvar(main = factvar, model = \"iid\",constr = FALSE, fixed=TRUE)",                                                      
                             "factvar_phi(main = factvar_phi, model = \"iid\", initial = -10, fixed = TRUE)",                                            
                             "numvar_intercept(1)",                                                                                                      
@@ -385,7 +451,8 @@ test_that('makeComponents is able to make the correct components for all the pro
                                         marks = c('numvar', 'factvar', 'binommark'), temporalmodel = deparse(list(model = "ar1")), speciesindependent = FALSE,
                                         multinomnames = 'factvar', pointcovariates = 'pointcov', marksspatial = TRUE, offsetname = NULL,
                                         speciesname = 'species', covariatenames = 'spatcovs', temporalname = 'temp', speciesspatial = 'individual',
-                                        covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel)
+                                        covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel,
+                                        biasformula = NULL, covariateformula = NULL, marksCopy = list(PO = c('numvar', 'factvar'), PA = 'binommark'))
     
     expect_setequal(compsData,
                     c("shared_spatial(main = geometry, model = shared_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",
@@ -400,13 +467,110 @@ test_that('makeComponents is able to make the correct components for all the pro
                       "fish1_spatcovs(main = fish1_spatcovs, model = \"numeric\")",                                                               
                       "bird2_spatcovs(main = bird2_spatcovs, model = \"numeric\")",                                                               
                       "bird1_spatcovs(main = bird1_spatcovs, model = \"numeric\")",                                                               
-                      "pointcov",                                                                                                                 
+                      "pointcov",
+                      "PO_numvar_spatial(main = geometry, copy = \"shared_spatial\")",
+                      "PO_factvar_spatial(main = geometry, copy = \"shared_spatial\")",
+                      "PA_binommark_spatial(main = geometry, copy = \"shared_spatial\")", 
                       'PO_intercept(1)', 'PA_intercept(1)',
                       "factvar(main = factvar, model = \"iid\",constr = FALSE, fixed=TRUE)",                                                      
                       "factvar_phi(main = factvar_phi, model = \"iid\", initial = -10, fixed = TRUE)",                                            
                       "numvar_intercept(1)",                                                                                                      
                       "binommark_intercept(1)"))
     
+    #Check biasFormula
+    compsBias <- Check$makeComponents(spatial = 'shared', intercepts = FALSE, datanames = c('PO','PA'), marksintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
+                                  marks = c('numvar', 'factvar', 'binommark'), temporalmodel = deparse(list(model = "ar1")), speciesindependent = FALSE,
+                                  multinomnames = 'factvar', pointcovariates = 'pointcov', marksspatial = TRUE, offsetname = NULL,
+                                  speciesname = 'species', covariatenames = 'spatcovs', temporalname = 'temp', speciesspatial = 'individual',
+                                  covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel,
+                                  biasformula = ~ BiasCov, covariateformula = NULL, marksCopy = list(PO = c('numvar', 'factvar'), PA = 'binommark'))
+    expect_setequal(compsBias,
+                    c("shared_spatial(main = geometry, model = shared_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",
+                      "fish2_PO_spatial(main = geometry, model = fish2_PO_field)",                                                                   
+                      "fish1_PO_spatial(main = geometry, model = fish1_PO_field)",                                                                   
+                      "bird2_PA_spatial(main = geometry, model = bird2_PA_field)",                                                                   
+                      "bird1_PA_spatial(main = geometry, model = bird1_PA_field)",                                                                   
+                      "numvar_spatial(main = geometry, model = numvar_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))"  ,                                                                 
+                      "factvar_spatial(main = geometry, model = factvar_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",                                                               
+                      "binommark_spatial(main = geometry, model = binommark_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",                                                           
+                      "fish2_spatcovs(main = fish2_spatcovs, model = \"numeric\")",                                                               
+                      "fish1_spatcovs(main = fish1_spatcovs, model = \"numeric\")",                                                               
+                      "bird2_spatcovs(main = bird2_spatcovs, model = \"numeric\")",                                                               
+                      "bird1_spatcovs(main = bird1_spatcovs, model = \"numeric\")",                                                               
+                      "pointcov",                                                                                                                 
+                      "fish2_intercept(1)",                                                                                                       
+                      "fish1_intercept(1)",                                                                                                       
+                      "bird2_intercept(1)",                                                                                                      
+                      "bird1_intercept(1)",
+                      "PO_numvar_spatial(main = geometry, copy = \"shared_spatial\")",
+                      "PO_factvar_spatial(main = geometry, copy = \"shared_spatial\")",
+                      "PA_binommark_spatial(main = geometry, copy = \"shared_spatial\")",
+                      "factvar(main = factvar, model = \"iid\",constr = FALSE, fixed=TRUE)",                                                      
+                      "factvar_phi(main = factvar_phi, model = \"iid\", initial = -10, fixed = TRUE)",                                            
+                      "numvar_intercept(1)",                                                                                                      
+                      "binommark_intercept(1)",
+                      "Bias__Effects__Comps(main = ~BiasCov - 1, model = \"fixed\")",
+                      "Bias__Effects__Comps(main = ~BiasCov - 1, model = \"fixed\")",
+                      "Bias__Effects__Comps(main = ~BiasCov - 1, model = \"fixed\")",
+                      "Bias__Effects__Comps(main = ~BiasCov - 1, model = \"fixed\")"))
+    
+    #Check covariateFormula
+    compsCov <- Check$makeComponents(spatial = 'shared', intercepts = FALSE, datanames = c('PO','PA'), marksintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
+                                      marks = c('numvar', 'factvar', 'binommark'), temporalmodel = deparse(list(model = "ar1")), speciesindependent = FALSE,
+                                      multinomnames = 'factvar', pointcovariates = 'pointcov', marksspatial = TRUE, offsetname = NULL,
+                                      speciesname = 'species', covariatenames = 'spatcovs', temporalname = 'temp', speciesspatial = 'individual',
+                                      covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel,
+                                      biasformula = NULL, covariateformula = ~ Var + I(Var^2), marksCopy = list(PO = c('numvar', 'factvar'), PA = 'binommark'))
+    expect_setequal(compsCov,
+                    c("shared_spatial(main = geometry, model = shared_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",
+                      "fish2_PO_spatial(main = geometry, model = fish2_PO_field)",                                                                   
+                      "fish1_PO_spatial(main = geometry, model = fish1_PO_field)",                                                                   
+                      "bird2_PA_spatial(main = geometry, model = bird2_PA_field)",                                                                   
+                      "bird1_PA_spatial(main = geometry, model = bird1_PA_field)",                                                                   
+                      "numvar_spatial(main = geometry, model = numvar_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))"  ,                                                                 
+                      "factvar_spatial(main = geometry, model = factvar_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",                                                               
+                      "binommark_spatial(main = geometry, model = binommark_field, group = temp, ngroup = 2, control.group = list(model = \"ar1\"))",                                                           
+                      "fish2_Fixed__Effects__Comps(main = ~fish2_Var + I(fish2_Var^2) - 1, model = \"fixed\")",
+                      "fish1_Fixed__Effects__Comps(main = ~fish1_Var + I(fish1_Var^2) - 1, model = \"fixed\")",
+                      "bird1_Fixed__Effects__Comps(main = ~bird1_Var + I(bird1_Var^2) - 1, model = \"fixed\")",
+                      "bird2_Fixed__Effects__Comps(main = ~bird2_Var + I(bird2_Var^2) - 1, model = \"fixed\")", 
+                      "pointcov",                                                                                                                 
+                      "fish2_intercept(1)",                                                                                                       
+                      "fish1_intercept(1)",                                                                                                       
+                      "bird2_intercept(1)",                                                                                                      
+                      "bird1_intercept(1)", 
+                      "PO_numvar_spatial(main = geometry, copy = \"shared_spatial\")",
+                      "PO_factvar_spatial(main = geometry, copy = \"shared_spatial\")",
+                      "PA_binommark_spatial(main = geometry, copy = \"shared_spatial\")",
+                      "factvar(main = factvar, model = \"iid\",constr = FALSE, fixed=TRUE)",                                                      
+                      "factvar_phi(main = factvar_phi, model = \"iid\", initial = -10, fixed = TRUE)",                                            
+                      "numvar_intercept(1)",                                                                                                      
+                      "binommark_intercept(1)"))
+    
+    compsCorrel <- Check$makeComponents(spatial = 'correlate', intercepts = FALSE, datanames = c('PO','PA'), marksintercept = TRUE, speciesintercept = FALSE, speciesenvironment = TRUE,
+                                     marks = NULL, temporalmodel = deparse(list(model = "ar1")), speciesindependent = FALSE,
+                                     multinomnames = 'factvar', pointcovariates = 'pointcov', marksspatial = TRUE, offsetname = NULL,
+                                     speciesname = 'species', covariatenames = 'spatcovs', temporalname = NULL, speciesspatial = 'individual',
+                                     covariateclass = 'numeric', numtime = 2, copymodel = Check$.__enclos_env__$private$copyModel,
+                                     biasformula = NULL, covariateformula = NULL)
+    
+    expect_setequal(compsCorrel, c("shared_spatial(main = geometry, model = shared_field, group = ._dataset_index_var_., control.group = list(model = \"exchangeable\"))",
+                                   "fish2_PO_spatial(main = geometry, model = fish2_PO_field)",
+                                   "fish1_PO_spatial(main = geometry, model = fish1_PO_field)", 
+                                   "bird1_PA_spatial(main = geometry, model = bird1_PA_field)", 
+                                   "bird2_PA_spatial(main = geometry, model = bird2_PA_field)",
+                                   "fish2_spatcovs(main = fish2_spatcovs, model = \"numeric\")",
+                                   "fish1_spatcovs(main = fish1_spatcovs, model = \"numeric\")",
+                                   "bird1_spatcovs(main = bird1_spatcovs, model = \"numeric\")", 
+                                   "bird2_spatcovs(main = bird2_spatcovs, model = \"numeric\")", 
+                                   "pointcov",
+                                   "factvar(main = factvar, model = \"iid\",constr = FALSE, fixed=TRUE)",
+                                   "factvar_phi(main = factvar_phi, model = \"iid\", initial = -10, fixed = TRUE)",
+                                   "fish2_intercept(1)",
+                                   "fish1_intercept(1)", 
+                                   "bird1_intercept(1)", 
+                                   "bird2_intercept(1)" ))
+    
             
           })
 
